@@ -24,9 +24,9 @@ use warnings;
 
 use Carp qw(croak);
 
-use Lintian::Output qw(debug_msg);
-use Text_utils qw(dtml_to_html dtml_to_text split_paragraphs wrap_paragraphs);
-use Util qw(fail read_dpkg_control);
+use Lintian::Data;
+use Lintian::Tag::TextUtil qw(dtml_to_html dtml_to_text split_paragraphs wrap_paragraphs);
+use Lintian::Util qw(fail);
 
 # The URL to a web man page service.  NAME is replaced by the man page
 # name and SECTION with the section to form a valid URL.  This is used
@@ -35,13 +35,9 @@ use Util qw(fail read_dpkg_control);
 our $MANURL
     = 'http://manpages.debian.net/cgi-bin/man.cgi?query=NAME&sektion=SECTION';
 
-# Stores the parsed tag information for all known tags.  Loaded the first
-# time new() is called.
-our %INFO;
-
 # Stores the parsed manual reference data.  Loaded the first time info()
 # is called.
-our %MANUALS;
+our $MANUALS = Lintian::Data->new ('output/manual-references', qr/::/, \&_load_manual_data);
 
 # Map severity/certainty levels to tag codes.
 our %CODES = (
@@ -75,57 +71,40 @@ metadata elements or to format the tag description.
 
 =over 4
 
-=item new(TAG)
+=item new(HASH, SCRIPT_NAME, SCRIPT_TYPE)
 
-Creates a new Lintian::Tag::Info object for the given TAG.  Returns undef
-if the tag is unknown and throws an exception if there is a parse error
-reading the check description files or if TAG is not specified.
-
-The first time this method is called, all tag metadata will be loaded into
-a memory cache.  This information will be used to satisfy all subsequent
-Lintian::Tag::Info object creation, avoiding multiple file reads.  This
-however means that a running Lintian process will not notice changes to
-tag metadata on disk.
+Creates a new Lintian::Tag:Info.
 
 =cut
 
-# Load all tag data into the %INFO hash.  Called by new() if %INFO is
-# empty and hence called the first time new() is called.
-sub _load_tag_data {
-    my $root = $ENV{LINTIAN_ROOT} || '/usr/share/lintian';
-    for my $desc (<$root/checks/*.desc>) {
-        debug_msg(2, "Reading checker description file $desc ...");
-        my ($header, @tags) = read_dpkg_control($desc);
-        unless ($header->{'check-script'}) {
-            fail("missing Check-Script field in $desc");
-        }
-        for my $tag (@tags) {
-            unless ($tag->{tag}) {
-                fail("missing Tag field in $desc");
-            }
-            $tag->{info} = '' unless exists($tag->{info});
-            $tag->{script} = $header->{'check-script'};
-            $tag->{'script-type'} = $header->{'type'};
-            $tag->{'effective-severity'} = $tag->{severity};
-            $INFO{$tag->{tag}} = $tag;
-        }
-    }
-}
-
-# Create a new object for the given tag.  We just use the hash created by
-# read_dpkg_control as the object, which means we slowly bless the objects
-# in %INFO as we return them.
 sub new {
-    my ($class, $tag) = @_;
-    croak('no tag specified') unless $tag;
-    _load_tag_data() unless %INFO;
-    if ($INFO{$tag}) {
-        my $self = $INFO{$tag};
-        bless($self, $class) unless ref($self) eq $class;
-        return $self;
-    } else {
-        return;
+    my ($class, $tag, $sn, $st) = @_;
+    my %copy;
+    my $self;
+    my $tagname;
+    croak 'no tag specified' unless $tag;
+    %copy = %$tag;
+    $self = \%copy;
+    croak "Missing Tag field" unless $self->{'tag'};
+    $tagname = $self->{'tag'};
+    croak "Missing Severity field for $tagname" unless $self->{'severity'};
+    croak "Missing Certainty field for $tagname" unless $self->{'certainty'};
+    $self->{'info'} = '' unless $self->{'info'};
+    $self->{'script'} = $sn;
+    $self->{'script-type'} = $st;
+    $self->{'effective-severity'} = $self->{severity};
+
+    bless $self, $class;
+
+    # Check the tag has a code - if it doesn't, either the severity or
+    # certainty is wrong (or we introduced a new one but forgot to add
+    # it to %CODES).
+    unless ($self->code) {
+        croak "Cannot determine the code of $tagname (severity:"
+            . " $self->{'severity'}, certainity: $self->{'certainty'}).\n";
     }
+
+    return $self;
 }
 
 =back
@@ -174,26 +153,17 @@ formatted output.
 
 =cut
 
-# Load manual reference data into %MANUALS.  This information doesn't have
-# a single unique key and has multiple data values per key, so we don't
-# try to use the Lintian::Data interface.  Instead, we read a file
-# delimited by double colons.  We do use a path similar to Lintian::Data
-# to keep such files in the same general location.
+# Parse manual reference data from the data file.
 sub _load_manual_data {
-    my $root = $ENV{LINTIAN_ROOT} || '/usr/share/lintian';
-    open(REFS, '<', "$root/data/output/manual-references")
-        or fail("can't open $root/data/output/manual-references: $!");
-    local $_;
-    while (<REFS>) {
-        chomp;
-        next if /^\#/;
-        next if /^\s*$/;
-        next unless /^(.+?)::(.*?)::(.+?)::(.*?)$/;
-        my ($manual, $section, $title, $url) = split('::');
-        $MANUALS{$manual}{$section}{title} = $title;
-        $MANUALS{$manual}{$section}{url} = $url;
+    my ($key, $rawvalue, $pval) = @_;
+    my ($section, $title, $url) = split m/::/, $rawvalue, 3;
+    my $ret = undef;
+    if (not defined $pval) {
+        $ret = $pval = {};
     }
-    close REFS;
+    $pval->{$section}{title} = $title;
+    $pval->{$section}{url} = $url;
+    return $ret;
 }
 
 # Format a reference to a manual in the HTML that Lintian uses internally
@@ -202,12 +172,12 @@ sub _load_manual_data {
 # argument isn't a known manual.
 sub _manual_reference {
     my ($manual, $section) = @_;
-    _load_manual_data unless %MANUALS;
-    return '' unless exists $MANUALS{$manual}{''};
+    return '' unless $MANUALS->known ($manual);
 
+    my $man = $MANUALS->value ($manual);
     # Start with the reference to the overall manual.
-    my $title = $MANUALS{$manual}{''}{title};
-    my $url   = $MANUALS{$manual}{''}{url};
+    my $title = $man->{''}{title};
+    my $url   = $man->{''}{url};
     my $text  = $url ? qq(<a href="$url">$title</a>) : $title;
 
     # Add the section information, if present, and a direct link to that
@@ -219,9 +189,9 @@ sub _manual_reference {
     } elsif ($section and $section =~ /^[A-Z\d.]+$/) {
         $text .= " section $section";
     }
-    if ($section and exists $MANUALS{$manual}{$section}) {
-        my $sec_title = $MANUALS{$manual}{$section}{title};
-        my $sec_url   = $MANUALS{$manual}{$section}{url};
+    if ($section and exists $man->{$section}) {
+        my $sec_title = $man->{$section}{title};
+        my $sec_url   = $man->{$section}{url};
         $text .= $sec_url ? qq[ (<a href="$sec_url">$sec_title</a>)] : qq[ ($sec_title)];
     }
 
@@ -280,20 +250,19 @@ sub description {
     my $info = $self->{info};
     $info =~ s/\n[ \t]/\n/g;
     my @text = split_paragraphs($info);
+    my $severity = $self->severity;
+    my $certainty = $self->certainty;
+
     if ($self->{ref}) {
         push(@text, '', _format_reference($self->{ref}));
     }
-    if ($self->{severity} and $self->{certainty}) {
-        my $severity = $self->{severity};
-        my $certainty = $self->{certainty};
-        push(@text, '', "Severity: $severity, Certainty: $certainty");
-    }
+    push(@text, '', "Severity: $severity, Certainty: $certainty");
     if ($self->{script} and $self->{'script-type'}){
         my $script = $self->{script};
         my $stype = $self->{'script-type'};
         push(@text, '', "Check: $script, Type: $stype");
     }
-    if ($self->{experimental}) {
+    if ($self->experimental) {
         push(@text, '',
              'This tag is marked experimental, which means that the code that'
              . ' generates it is not as well-tested as the rest of Lintian'
@@ -345,6 +314,7 @@ Modifies the effective severity of the tag.
 
 sub set_severity{
     my ($self, $sev) = @_;
+    croak "Unknown severity $sev.\n" unless exists $CODES{$sev};
     $self->{'effective-severity'} = $sev;
 }
 
@@ -476,4 +446,4 @@ lintian(1)
 # indent-tabs-mode: nil
 # cperl-indent-level: 4
 # End:
-# vim: syntax=perl sw=4 sts=4 ts=4 et shiftround
+# vim: syntax=perl sw=4 sts=4 sr et
